@@ -25,23 +25,35 @@ typedef struct
 
 #include "boot_embedded_files.c"
 
+#define EXTRACT_FAIL    0
+#define EXTRACT_OK      1
+#define EXTRACT_ALREADY 2
 
+/* extract EMB_FILE to file STMPDIR/EXT_NAME and set *EXT_PATH to the latter;
+ * return EXTRACT_ALREADY if the extracted file already exists (and has the
+ * expected size), EXTRACT_OK if successful, EXTRACT_FAIL otherwise
+ */
 static 
 int extract_embedded_file(embedded_file_t *emb_file, const char* ext_name, const char* stmpdir, char** ext_path) {
     int fd;
     chunk_t *chunk;
+    struct stat statbuf;
 
     *ext_path = malloc(strlen(stmpdir) + 1 + strlen(ext_name) + 1);
     sprintf(*ext_path, "%s/%s", stmpdir, ext_name);
 
     fd = open(*ext_path, O_CREAT | O_EXCL | O_WRONLY | OPEN_O_BINARY, 0755);
     if ( fd == -1 ) {
-        struct stat statbuf;
-        if ( errno == EEXIST 
-             && par_lstat(*ext_path, &statbuf) == 0 
-             && statbuf.st_size == emb_file->size )
-            return 1;           /* file already exists and has the expected size */
-        return 0;
+        if ( errno != EEXIST ) return EXTRACT_FAIL;
+
+        if (par_lstat(*ext_path, &statbuf) == 0 
+            && statbuf.st_size == emb_file->size )
+            /* file already exists and has the expected size */
+            return EXTRACT_ALREADY;           
+
+        /* corrupted file? re-try writing it */
+        fd = open(*ext_path, O_CREAT | O_WRONLY | OPEN_O_BINARY, 0755);
+        if ( fd == -1 ) return EXTRACT_FAIL;
     }
 
     chunk = emb_file->chunks;
@@ -52,10 +64,10 @@ int extract_embedded_file(embedded_file_t *emb_file, const char* ext_name, const
         chunk++;
     }
     if (close(fd) == -1)
-        return 0;
+        return EXTRACT_FAIL;
 
     chmod(*ext_path, 0755);
-    return 1;
+    return EXTRACT_OK;
 }
 
 
@@ -100,7 +112,7 @@ void seek_to_subsystem( int fd ) {
 
 int main ( int argc, char **argv, char **env )
 {
-    int i;
+    int rc, i;
     char *stmpdir;
     embedded_file_t *emb_file;
     char *my_file;
@@ -123,8 +135,8 @@ typedef BOOL (WINAPI *pALLOW)(DWORD);
     stmpdir = par_mktmpdir( argv );	
     if ( !stmpdir ) DIE;        /* error message has already been printed */
 
-    i = my_mkdir(stmpdir, 0700);
-    if ( i == -1 && errno != EEXIST) {
+    rc = my_mkdir(stmpdir, 0700);
+    if ( rc == -1 && errno != EEXIST) {
 	fprintf(stderr, "%s: creation of private cache subdirectory %s failed (errno= %i)\n", 
                         argv[0], stmpdir, errno);
  	DIE;
@@ -133,12 +145,15 @@ typedef BOOL (WINAPI *pALLOW)(DWORD);
     /* extract embedded_files[0] (i.e. the custom Perl interpreter) 
      * into stmpdir (but under the same basename as argv[0]) */
     my_prog = par_findprog(argv[0], strdup(par_getenv("PATH")));
-    if (!extract_embedded_file(embedded_files, par_basename(my_prog), stmpdir, &my_perl)) {
+    rc = extract_embedded_file(embedded_files, par_basename(my_prog), stmpdir, &my_perl);
+    if (rc == EXTRACT_FAIL) {
         fprintf(stderr, "%s: extraction of %s (custom Perl interpreter) failed (errno=%i)\n", 
                             argv[0], my_perl, errno);
         DIE;
     }
 
+    if (rc == EXTRACT_OK)       /* i.e. file didn't already exist */
+    {
 #ifdef __hpux
         {
             /* HPUX will only honour SHLIB_PATH if the executable is specially marked */
@@ -168,11 +183,12 @@ typedef BOOL (WINAPI *pALLOW)(DWORD);
             close(fd);                  // CHECK != -1
         }
 #endif
+    }
 
     /* extract the rest of embedded_files into stmpdir */
     emb_file = embedded_files + 1;
     while (emb_file->name) {
-        if (!extract_embedded_file(emb_file, emb_file->name, stmpdir, &my_file)) {
+        if (extract_embedded_file(emb_file, emb_file->name, stmpdir, &my_file) == EXTRACT_FAIL) {
             fprintf(stderr, "%s: extraction of %s failed (errno=%i)\n", 
                                 argv[0], my_file, errno);
             DIE;
@@ -201,10 +217,10 @@ typedef BOOL (WINAPI *pALLOW)(DWORD);
     }
 
     par_setenv("PAR_SPAWNED", "1");
-    i = spawnvpe(P_WAIT, my_perl, (const char* const*)argv, (const char* const*)environ);
+    rc = spawnvpe(P_WAIT, my_perl, (const char* const*)argv, (const char* const*)environ);
 
     par_cleanup(stmpdir);
-    exit(i);
+    exit(rc);
 #else
     execvp(my_perl, argv);
     DIE;
